@@ -7,7 +7,7 @@ class CmModelConnection {
   FILE *io_stream;
   int master;
   XtInputId input_id;
-  XtWorkProcId work_id;
+  XtInputId output_id;
   int child_pid;
   Boolean is_blocked;
   Boolean has_echo;
@@ -15,6 +15,7 @@ class CmModelConnection {
   XtAppContext app_context;
   XtInputCallbackProc connection_callback;
   XtPointer connection_client_data;
+  int send_signal(int);
 public:
   CmModelConnection(const CmString &, XtAppContext);
   ~CmModelConnection();
@@ -24,8 +25,8 @@ public:
   void SetEcho(Boolean);
   int SendSignal(int);
   void Interrupt();
-  friend int
-    CmModelConnectionWorkProc(CmModelConnection*);
+  friend void
+    CmModelConnectionWorkProc(CmModelConnection*, int *, XtInputId *);
   friend void
     CmModelConnectionCallback(CmModelConnection*, int *, XtInputId *);
 };
