@@ -875,10 +875,10 @@ char **argv;
 
 	int result;		/* Tcl result */
 
-	time_t current_time;	/* current time (when we last looked)*/
+	time_t current_time = 0;/* current time (when we last looked)*/
 	time_t end_time;	/* future time at which to give up */
 
-	struct f *last_f;	/* for differentiating when we have multiple fs */
+	struct f *last_f;	/* for differentiating when multiple f's */
 				/* to print out better debugging messages */
 	int last_case;		/* as above but for case */
 	int do_flush;		/* if anything queued up, should be flushed */
@@ -949,20 +949,22 @@ char **argv;
 		flush_streams();
 	}
 
-	time(&current_time);
-	end_time = current_time + timeout;
+	/* code works even if ==, however this avoids pointless system call */
+	if (timeout != TIME_INFINITY) {
+		time(&current_time);
+		end_time = current_time + timeout;
+	}
 
 	for (;;) {
 	     /* if first time thru, try to match against */
 	     /* old buffered data before reading more */
 	    if (m != SPAWN_ID_ANY) {
-		int remtime;	/* max time to read().  This accounts for */
-			/* systems which wait in ready, and ancient ones */
-			/* which can't, and have to wait in read() */
+		int remtime = end_time - current_time;  /* max time to read */
+		/* This accounts for systems which wait in ready, and */
+		/* ancient ones which can't, and have to wait in read() */
 	
-		remtime = end_time - current_time;
-		
-		if (timeout == TIME_INFINITY || current_time < end_time) {
+		if (timeout == TIME_INFINITY || remtime > 0) {
+			if (timeout == TIME_INFINITY) remtime = -1;
 			cc = i_read(masters,mcount,&m,remtime);
 		} else {
 			cc = EXP_TIMEOUT;
@@ -1052,7 +1054,8 @@ char **argv;
 
 	     /* if we've not yet actually read anything, don't update time */
 	     /* this forces at least one read to be done if timeout > 0 */
-	     if (m != SPAWN_ID_ANY) time(&current_time);
+	     if (m != SPAWN_ID_ANY)
+		if (timeout != TIME_INFINITY) time(&current_time);
 
 	     m = 0; /* as long as we assign m any non-SPAWN_ID_ANY value */
 		/* we will be assured that the next time through the loop */
@@ -1506,9 +1509,13 @@ init_trap()
 
 #ifdef HPUX
 	/* initially forced to catch & discard SIGCLD to collect wait status */
-	(void) Tcl_Eval(interp,"trap SIG_DFL SIGCLD",0,(char **)0);
+	(void) Tcl_Eval(interp,"trap SIG_DFL SIGCHLD",0,(char **)0);
 	/* no point in checking for errors here, since it is so early on */
 	/* something else will be sure to fail before application begins */
+
+	/* note that SIGCHLD is used rather than SIGCLD since HPUX defines */
+	/* them both, but expect can only handle one, and it handles the */
+	/* "wrong" one, first */
 #endif
 }
 
@@ -1584,7 +1591,9 @@ int sig;
 			/* this code */
 			if (sig == SIGCLD) {
 				int i, pid;
-					
+				int status;
+				extern int fd_max;
+
 				pid = wait(&status);
 				for (i=0;i<=fd_max;i++) {
 					if (fs[i].pid == pid) break;
